@@ -1,6 +1,6 @@
 /* ModelicaIO.c - Array I/O functions
 
-   Copyright (C) 2016-2018, Modelica Association and ESI ITI GmbH
+   Copyright (C) 2016-2019, Modelica Association and contributors
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -13,6 +13,10 @@
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
 
+   3. Neither the name of the copyright holder nor the names of its
+      contributors may be used to endorse or promote products derived from
+      this software without specific prior written permission.
+
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -607,8 +611,9 @@ static int IsNumber(char* token) {
     int foundExponentSign = 0;
     int foundExponent = 0;
     int foundDec = 0;
+    int foundDigit = 0;
     int isNumber = 1;
-    int k = 0;
+    int k;
 
     if (token[0] == '-' || token[0] == '+') {
         k = 1;
@@ -619,6 +624,7 @@ static int IsNumber(char* token) {
     while (token[k] != '\0') {
         if (token[k] >= '0' && token[k] <= '9') {
             k++;
+            foundDigit++;
         }
         else if (token[k] == '.' && foundDec == 0 &&
             foundExponent == 0 && foundExponentSign == 0) {
@@ -626,8 +632,9 @@ static int IsNumber(char* token) {
             k++;
         }
         else if ((token[k] == 'e' || token[k] == 'E') &&
-            foundExponent == 0) {
+            foundExponent == 0 && foundDigit > 0) {
             foundExponent = 1;
+            foundDigit = 0;
             k++;
         }
         else if ((token[k] == '-' || token[k] == '+') &&
@@ -640,7 +647,7 @@ static int IsNumber(char* token) {
             break;
         }
     }
-    return isNumber;
+    return isNumber && foundDigit > 0;
 }
 
 static double* readTxtTable(_In_z_ const char* fileName, _In_z_ const char* tableName,
@@ -913,6 +920,7 @@ static double* readTxtTable(_In_z_ const char* fileName, _In_z_ const char* tabl
                     unsigned long lineNoPartial = lineNo;
                     int tableReadPartial = 0;
                     while (readLine(&buf, &bufLen, fp) == 0) {
+                        k = 0;
                         lineNoPartial++;
                         /* Ignore leading white space */
                         while (k < bufLen - 1) {
