@@ -1,6 +1,6 @@
 /* ED_MATFile.h - MAT functions header
  *
- * Copyright (C) 2015-2016, tbeu
+ * Copyright (C) 2015-2019, tbeu
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,7 @@
 void* ED_createMAT(const char* fileName, int verbose);
 void ED_destroyMAT(void* _mat);
 void ED_getDoubleArray2DFromMAT(void* _mat, const char* varName, double* a, size_t m, size_t n);
-void ED_getStringArray1DFromMAT(void* _mat, const char* varName, const char* string[], size_t m);
+void ED_getStringArray1DFromMAT(void* _mat, const char* varName, const char** a, size_t m);
+void ED_getArray2DDimensionsFromMAT(void* _mat, const char* varName, int* m, int* n);
 
 #endif
